from __future__ import annotations
from dataclasses import dataclass, field
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.base_request_configuration import RequestConfiguration
from kiota_abstractions.default_query_parameters import QueryParameters
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.method import Method
from kiota_abstractions.request_adapter import RequestAdapter
from kiota_abstractions.request_information import RequestInformation
from kiota_abstractions.request_option import RequestOption
from kiota_abstractions.serialization import Parsable, ParsableFactory
from typing import Any, Callable, Dict, List, Optional, TYPE_CHECKING, Union
from warnings import warn

if TYPE_CHECKING:
    from ......models.o_data_errors.o_data_error import ODataError
    from ......models.webauthn_credential_creation_options import WebauthnCredentialCreationOptions

class CreationOptionsWithChallengeTimeoutInMinutesRequestBuilder(BaseRequestBuilder):
    """
    Provides operations to call the creationOptions method.
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Union[str, Dict[str, Any]]) -> None:
        """
        Instantiates a new CreationOptionsWithChallengeTimeoutInMinutesRequestBuilder and sets the default values.
        param path_parameters: The raw url or the url-template parameters for the request.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        super().__init__(request_adapter, "{+baseurl}/users/{user%2Did}/authentication/fido2Methods/creationOptions(challengeTimeoutInMinutes=@challengeTimeoutInMinutes){?challengeTimeoutInMinutes*}", path_parameters)
    
    async def get(self,request_configuration: Optional[RequestConfiguration[CreationOptionsWithChallengeTimeoutInMinutesRequestBuilderGetQueryParameters]] = None) -> Optional[WebauthnCredentialCreationOptions]:
        """
        Retrieve creation options required to generate and register a Microsoft Entra ID-compatible passkey. Self-service operations aren't supported.  
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: Optional[WebauthnCredentialCreationOptions]
        Find more info here: https://learn.microsoft.com/graph/api/fido2authenticationmethod-creationoptions?view=graph-rest-beta
        """
        warn(" as of 2024-07/PrivatePreview:copilotExportAPI", DeprecationWarning)
        request_info = self.to_get_request_information(
            request_configuration
        )
        from ......models.o_data_errors.o_data_error import ODataError

        error_mapping: Dict[str, type[ParsableFactory]] = {
            "XXX": ODataError,
        }
        if not self.request_adapter:
            raise Exception("Http core is null") 
        from ......models.webauthn_credential_creation_options import WebauthnCredentialCreationOptions

        return await self.request_adapter.send_async(request_info, WebauthnCredentialCreationOptions, error_mapping)
    
    def to_get_request_information(self,request_configuration: Optional[RequestConfiguration[CreationOptionsWithChallengeTimeoutInMinutesRequestBuilderGetQueryParameters]] = None) -> RequestInformation:
        """
        Retrieve creation options required to generate and register a Microsoft Entra ID-compatible passkey. Self-service operations aren't supported.  
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        warn(" as of 2024-07/PrivatePreview:copilotExportAPI", DeprecationWarning)
        request_info = RequestInformation(Method.GET, self.url_template, self.path_parameters)
        request_info.configure(request_configuration)
        request_info.headers.try_add("Accept", "application/json")
        return request_info
    
    def with_url(self,raw_url: str) -> CreationOptionsWithChallengeTimeoutInMinutesRequestBuilder:
        """
        Returns a request builder with the provided arbitrary URL. Using this method means any other path or query parameters are ignored.
        param raw_url: The raw URL to use for the request builder.
        Returns: CreationOptionsWithChallengeTimeoutInMinutesRequestBuilder
        """
        warn(" as of 2024-07/PrivatePreview:copilotExportAPI", DeprecationWarning)
        if raw_url is None:
            raise TypeError("raw_url cannot be null.")
        return CreationOptionsWithChallengeTimeoutInMinutesRequestBuilder(self.request_adapter, raw_url)
    
    @dataclass
    class CreationOptionsWithChallengeTimeoutInMinutesRequestBuilderGetQueryParameters():
        """
        Retrieve creation options required to generate and register a Microsoft Entra ID-compatible passkey. Self-service operations aren't supported.  
        """
        def get_query_parameter(self,original_name: str) -> str:
            """
            Maps the query parameters names to their encoded names for the URI template parsing.
            param original_name: The original query parameter name in the class.
            Returns: str
            """
            if original_name is None:
                raise TypeError("original_name cannot be null.")
            if original_name == "challenge_timeout_in_minutes":
                return "challengeTimeoutInMinutes"
            return original_name
        
        # Usage: challengeTimeoutInMinutes=@challengeTimeoutInMinutes
        challenge_timeout_in_minutes: Optional[int] = None

    
    @dataclass
    class CreationOptionsWithChallengeTimeoutInMinutesRequestBuilderGetRequestConfiguration(RequestConfiguration[CreationOptionsWithChallengeTimeoutInMinutesRequestBuilderGetQueryParameters]):
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        warn("This class is deprecated. Please use the generic RequestConfiguration class generated by the generator.", DeprecationWarning)
    

