from __future__ import annotations
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Callable, Dict, List, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .app_vulnerability_managed_device import AppVulnerabilityManagedDevice
    from .app_vulnerability_mobile_app import AppVulnerabilityMobileApp
    from .app_vulnerability_task_mitigation_type import AppVulnerabilityTaskMitigationType
    from .device_app_management_task import DeviceAppManagementTask

from .device_app_management_task import DeviceAppManagementTask

@dataclass
class AppVulnerabilityTask(DeviceAppManagementTask):
    """
    An app vulnerability task.
    """
    # The OdataType property
    odata_type: Optional[str] = "#microsoft.graph.appVulnerabilityTask"
    # The app name.
    app_name: Optional[str] = None
    # The app publisher.
    app_publisher: Optional[str] = None
    # The app version.
    app_version: Optional[str] = None
    # Information about the mitigation.
    insights: Optional[str] = None
    # The number of vulnerable devices.
    managed_device_count: Optional[int] = None
    # The vulnerable managed devices.
    managed_devices: Optional[List[AppVulnerabilityManagedDevice]] = None
    # Device app management task mitigation type.
    mitigation_type: Optional[AppVulnerabilityTaskMitigationType] = None
    # The number of vulnerable mobile apps.
    mobile_app_count: Optional[int] = None
    # The vulnerable mobile apps.
    mobile_apps: Optional[List[AppVulnerabilityMobileApp]] = None
    # The remediation steps.
    remediation: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> AppVulnerabilityTask:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: AppVulnerabilityTask
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return AppVulnerabilityTask()
    
    def get_field_deserializers(self,) -> Dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: Dict[str, Callable[[ParseNode], None]]
        """
        from .app_vulnerability_managed_device import AppVulnerabilityManagedDevice
        from .app_vulnerability_mobile_app import AppVulnerabilityMobileApp
        from .app_vulnerability_task_mitigation_type import AppVulnerabilityTaskMitigationType
        from .device_app_management_task import DeviceAppManagementTask

        from .app_vulnerability_managed_device import AppVulnerabilityManagedDevice
        from .app_vulnerability_mobile_app import AppVulnerabilityMobileApp
        from .app_vulnerability_task_mitigation_type import AppVulnerabilityTaskMitigationType
        from .device_app_management_task import DeviceAppManagementTask

        fields: Dict[str, Callable[[Any], None]] = {
            "appName": lambda n : setattr(self, 'app_name', n.get_str_value()),
            "appPublisher": lambda n : setattr(self, 'app_publisher', n.get_str_value()),
            "appVersion": lambda n : setattr(self, 'app_version', n.get_str_value()),
            "insights": lambda n : setattr(self, 'insights', n.get_str_value()),
            "managedDeviceCount": lambda n : setattr(self, 'managed_device_count', n.get_int_value()),
            "managedDevices": lambda n : setattr(self, 'managed_devices', n.get_collection_of_object_values(AppVulnerabilityManagedDevice)),
            "mitigationType": lambda n : setattr(self, 'mitigation_type', n.get_enum_value(AppVulnerabilityTaskMitigationType)),
            "mobileAppCount": lambda n : setattr(self, 'mobile_app_count', n.get_int_value()),
            "mobileApps": lambda n : setattr(self, 'mobile_apps', n.get_collection_of_object_values(AppVulnerabilityMobileApp)),
            "remediation": lambda n : setattr(self, 'remediation', n.get_str_value()),
        }
        super_fields = super().get_field_deserializers()
        fields.update(super_fields)
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        super().serialize(writer)
        writer.write_str_value("appName", self.app_name)
        writer.write_str_value("appPublisher", self.app_publisher)
        writer.write_str_value("appVersion", self.app_version)
        writer.write_str_value("insights", self.insights)
        writer.write_int_value("managedDeviceCount", self.managed_device_count)
        writer.write_collection_of_object_values("managedDevices", self.managed_devices)
        writer.write_enum_value("mitigationType", self.mitigation_type)
        writer.write_int_value("mobileAppCount", self.mobile_app_count)
        writer.write_collection_of_object_values("mobileApps", self.mobile_apps)
        writer.write_str_value("remediation", self.remediation)
    

